@@ -1,4 +1,5 @@
 import redis
+
 try:
     from django.utils.encoding import force_unicode
 except ImportError:  # Python 3.*
@@ -9,14 +10,30 @@ from redis_sessions import settings
 
 # Avoid new redis connection on each request
 
+if settings.SESSION_REDIS_SENTINEL_LIST is not None:
+    from redis.sentinel import Sentinel
 
-if settings.SESSION_REDIS_URL is not None:
-    redis_server = redis.StrictRedis.from_url(settings.SESSION_REDIS_URL)
+    redis_server = Sentinel(
+        settings.SESSION_REDIS_SENTINEL_LIST,
+        socket_timeout=settings.SESSION_REDIS_SOCKET_TIMEOUT,
+        retry_on_timeout=settings.SESSION_REDIS_RETRY_ON_TIMEOUT,
+        db=getattr(settings, 'SESSION_REDIS_DB', 0),
+        password=getattr(settings, 'SESSION_REDIS_PASSWORD', None)
+    ).master_for(settings.SESSION_REDIS_SENTINEL_MASTER_ALIAS)
+
+elif settings.SESSION_REDIS_URL is not None:
+
+    redis_server = redis.StrictRedis.from_url(
+        settings.SESSION_REDIS_URL,
+        socket_timeout=settings.SESSION_REDIS_SOCKET_TIMEOUT
+    )
 elif settings.SESSION_REDIS_UNIX_DOMAIN_SOCKET_PATH is None:
     
     redis_server = redis.StrictRedis(
         host=settings.SESSION_REDIS_HOST,
         port=settings.SESSION_REDIS_PORT,
+        socket_timeout=settings.SESSION_REDIS_SOCKET_TIMEOUT,
+        retry_on_timeout=settings.SESSION_REDIS_RETRY_ON_TIMEOUT,
         db=settings.SESSION_REDIS_DB,
         password=settings.SESSION_REDIS_PASSWORD
     )
@@ -24,6 +41,8 @@ else:
 
     redis_server = redis.StrictRedis(
         unix_socket_path=settings.SESSION_REDIS_UNIX_DOMAIN_SOCKET_PATH,
+        socket_timeout=settings.SESSION_REDIS_SOCKET_TIMEOUT,
+        retry_on_timeout=settings.SESSION_REDIS_RETRY_ON_TIMEOUT,
         db=settings.SESSION_REDIS_DB,
         password=settings.SESSION_REDIS_PASSWORD,
     )
@@ -45,7 +64,7 @@ class SessionStore(SessionBase):
             )
             return self.decode(force_unicode(session_data))
         except:
-            self.create()
+            self._session_key = None
             return {}
 
     def exists(self, session_key):
@@ -58,11 +77,14 @@ class SessionStore(SessionBase):
             try:
                 self.save(must_create=True)
             except CreateError:
+                # Key wasn't unique. Try again.
                 continue
             self.modified = True
             return
 
     def save(self, must_create=False):
+        if self.session_key is None:
+            return self.create()
         if must_create and self.exists(self._get_or_create_session_key()):
             raise CreateError
         data = self.encode(self._get_session(no_load=must_create))
